@@ -40,6 +40,7 @@
 #include <interfaces/ilanguage.h>
 #include <interfaces/ilanguagecontroller.h>
 #include <interfaces/foregroundlock.h>
+#include <interfaces/isession.h>
 
 #include <util/google/dense_hash_map>
 
@@ -146,10 +147,11 @@ class EnvironmentInformationRequest {
   void createItem(EnvironmentInformationItem* item) const {
     new (item) EnvironmentInformationItem(m_index, DUChainItemSystem::self().dynamicSize(*m_file->d_func()));
     Q_ASSERT(m_file->d_func()->m_dynamic);
-    DUChainItemSystem::self().copy(*m_file->d_func(), *(DUChainBaseData*)(((char*)item) + sizeof(EnvironmentInformationItem)), true);
-    Q_ASSERT((*(DUChainBaseData*)(((char*)item) + sizeof(EnvironmentInformationItem))).m_range == m_file->d_func()->m_range);
-    Q_ASSERT((*(DUChainBaseData*)(((char*)item) + sizeof(EnvironmentInformationItem))).classId == m_file->d_func()->classId);
-    Q_ASSERT((*(DUChainBaseData*)(((char*)item) + sizeof(EnvironmentInformationItem))).m_dynamic == false);
+    DUChainBaseData* data = reinterpret_cast<DUChainBaseData*>(reinterpret_cast<char*>(item) + sizeof(EnvironmentInformationItem));
+    DUChainItemSystem::self().copy(*m_file->d_func(), *data, true);
+    Q_ASSERT(data->m_range == m_file->d_func()->m_range);
+    Q_ASSERT(data->classId == m_file->d_func()->classId);
+    Q_ASSERT(data->m_dynamic == false);
   }
   
   static void destroy(EnvironmentInformationItem* item, KDevelop::AbstractItemRepository&) {
@@ -265,10 +267,11 @@ class DUChainPrivate
       }
 
       void stopThread() {
-        m_waitMutex.lock();
-        m_stopRunning = true;
-        m_wait.wakeAll(); //Wakes the thread up, so it notices it should exit
-        m_waitMutex.unlock();
+        {
+          QMutexLocker lock(&m_waitMutex);
+          m_stopRunning = true;
+          m_wait.wakeAll(); //Wakes the thread up, so it notices it should exit
+        }
         wait();
       }
 
@@ -278,9 +281,8 @@ class DUChainPrivate
           for(uint s = 0; s < cleanupEverySeconds; ++s) {
             if(m_stopRunning)
               break;
-            m_waitMutex.lock();
+            QMutexLocker lock(&m_waitMutex);
             m_wait.wait(&m_waitMutex, 1000);
-            m_waitMutex.unlock();
           }
           if(m_stopRunning)
             break;
@@ -327,7 +329,8 @@ public:
       ///@todo Solve this more duchain-like
       QFile f(globalItemRepositoryRegistry().path() + "/parsing_environment_data");
       bool opened = f.open(QIODevice::ReadOnly);
-      ParsingEnvironmentFile::m_staticData = (StaticParsingEnvironmentData*) new char[sizeof(StaticParsingEnvironmentData)];
+      ///FIXME: ugh, so ugly
+      ParsingEnvironmentFile::m_staticData = reinterpret_cast<StaticParsingEnvironmentData*>( new char[sizeof(StaticParsingEnvironmentData)]);
       if(opened) {
         kDebug() << "reading parsing-environment static data";
         //Read
@@ -357,6 +360,7 @@ public:
     m_cleanup->stopThread();
     delete m_cleanup;
     delete instance;
+    delete notifier;
   }
 
   void clear() {
@@ -463,12 +467,13 @@ public:
 
     info->makeDynamic(); //By doing this, we make sure the data is actually being destroyed in the destructor
 
-    m_chainsMutex.lock();
-    bool removed = m_fileEnvironmentInformations.remove(info->url(), info);
-    
-    bool removed2 = m_indexEnvironmentInformations.remove(info->indexedTopContext().index());
-    
-    m_chainsMutex.unlock();
+    bool removed = false;
+    bool removed2 = false;
+    {
+      QMutexLocker lock(&m_chainsMutex);
+      removed = m_fileEnvironmentInformations.remove(info->url(), info);
+      removed2 = m_indexEnvironmentInformations.remove(info->indexedTopContext().index());
+    }
     
     {
       //Remove it from the environment information lists if it was there
@@ -491,6 +496,8 @@ public:
       m_environmentInfo.deleteItem(index);
     }
 
+    Q_UNUSED(removed);
+    Q_UNUSED(removed2);
     Q_ASSERT(index || (removed && removed2));
     Q_ASSERT(!findInformation(info->indexedTopContext().index()));
   }
@@ -635,9 +642,7 @@ public:
           Q_ASSERT(index);
 
           EnvironmentInformationItem* item = const_cast<EnvironmentInformationItem*>(m_environmentInfo.itemFromIndex(index));
-          DUChainBaseData* theData = (DUChainBaseData*)(((char*)item) + sizeof(EnvironmentInformationItem));
-          static DUChainBaseData* dataCopy;
-          dataCopy = theData;
+          DUChainBaseData* theData = reinterpret_cast<DUChainBaseData*>(reinterpret_cast<char*>(item) + sizeof(EnvironmentInformationItem));
 
           Q_ASSERT(theData->m_range == file->d_func()->m_range);
           Q_ASSERT(theData->m_dynamic == false);
@@ -857,6 +862,7 @@ public:
         QFile f(globalItemRepositoryRegistry().path() + "/parsing_environment_data");
         bool opened = f.open(QIODevice::WriteOnly);
         Q_ASSERT(opened);
+        Q_UNUSED(opened);
         f.write((char*)ParsingEnvironmentFile::m_staticData, sizeof(StaticParsingEnvironmentData));
       }
       
@@ -867,6 +873,7 @@ public:
         QFile f(globalItemRepositoryRegistry().path() + "/available_top_context_indices");
         bool opened = f.open(QIODevice::WriteOnly);
         Q_ASSERT(opened);
+        Q_UNUSED(opened);
 
         f.write((char*)m_availableTopContextIndices.data(), m_availableTopContextIndices.size() * sizeof(uint));
       }
@@ -926,7 +933,10 @@ public:
     if(alreadyLoaded)
       return alreadyLoaded;
     
-    ParsingEnvironmentFile* ret = dynamic_cast<ParsingEnvironmentFile*>(DUChainItemSystem::self().create( (DUChainBaseData*)(((char*)&item) + sizeof(EnvironmentInformationItem)) ));
+    ///FIXME: ugly, and remove const_cast
+    ParsingEnvironmentFile* ret = dynamic_cast<ParsingEnvironmentFile*>(DUChainItemSystem::self().create(
+      const_cast<DUChainBaseData*>(reinterpret_cast<const DUChainBaseData*>(reinterpret_cast<const char*>(&item) + sizeof(EnvironmentInformationItem)))
+    ));
     if(ret) {
       Q_ASSERT(ret->d_func()->classId);
       Q_ASSERT(ret->indexedTopContext().index() == topContextIndex);
@@ -1100,16 +1110,12 @@ K_GLOBAL_STATIC(DUChainPrivate, sdDUChainPrivate)
 
 DUChain::DUChain()
 {
-  connect(QCoreApplication::instance(), SIGNAL(aboutToQuit()), this, SLOT(aboutToQuit()));
+  Q_ASSERT(ICore::self());
 
-  if(ICore::self()) {
-    Q_ASSERT(ICore::self()->documentController());
-    connect(ICore::self()->documentController(), SIGNAL(documentLoadedPrepare(KDevelop::IDocument*)), this, SLOT(documentLoadedPrepare(KDevelop::IDocument*)));
-    connect(ICore::self()->documentController(), SIGNAL(documentUrlChanged(KDevelop::IDocument*)), this, SLOT(documentRenamed(KDevelop::IDocument*)));
-    connect(ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), this, SLOT(documentActivated(KDevelop::IDocument*)));
-    connect(ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), this, SLOT(documentClosed(KDevelop::IDocument*)));
-    
-  }
+  connect(ICore::self()->documentController(), SIGNAL(documentLoadedPrepare(KDevelop::IDocument*)), this, SLOT(documentLoadedPrepare(KDevelop::IDocument*)));
+  connect(ICore::self()->documentController(), SIGNAL(documentUrlChanged(KDevelop::IDocument*)), this, SLOT(documentRenamed(KDevelop::IDocument*)));
+  connect(ICore::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), this, SLOT(documentActivated(KDevelop::IDocument*)));
+  connect(ICore::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), this, SLOT(documentClosed(KDevelop::IDocument*)));
 }
 
 DUChain::~DUChain()
@@ -1132,7 +1138,9 @@ extern void initReferenceCounting();
 void DUChain::initialize()
 {
   // Initialize the global item repository as first thing after loading the session
-  globalItemRepositoryRegistry();
+  Q_ASSERT(ICore::self());
+  Q_ASSERT(ICore::self()->activeSession());
+  ItemRepositoryRegistry::initialize(ICore::self()->activeSessionLock());
 
   initReferenceCounting();
 
@@ -1253,6 +1261,7 @@ void DUChain::addToEnvironmentManager( TopDUContext * chain ) {
     ///If this triggers, there has already been another environment-information registered for this top-context.
     ///removeFromEnvironmentManager should have been called before to remove the old environment-information.
     Q_ASSERT(alreadyHave == file.data());
+    Q_UNUSED(alreadyHave);
     return;
   }
 
@@ -1455,6 +1464,18 @@ QList<KUrl> DUChain::documents() const
   return ret;
 }
 
+QList<IndexedString> DUChain::indexedDocuments() const
+{
+  QMutexLocker l(&sdDUChainPrivate->m_chainsMutex);
+
+  QList<IndexedString> ret;
+  foreach(TopDUContext* top, sdDUChainPrivate->m_chainsByUrl.values()) {
+    ret << top->url();
+  }
+
+  return ret;
+}
+
 /*Q_SCRIPTABLE bool DUChain::updateContext(TopDUContext* topContext, TopDUContext::Features minFeatures, QObject* notifyReady) const
 {
   if( (topContext->features() & minFeatures) != minFeatures || (topContext->parsingEnvironmentFile() && topContext->parsingEnvironmentFile()->needsUpdate()) ) {
@@ -1477,7 +1498,7 @@ void DUChain::documentActivated(KDevelop::IDocument* doc)
   TopDUContext* ctx = DUChainUtils::standardContextForUrl(doc->url(), true);
   if(ctx && ctx->parsingEnvironmentFile())
     if(ctx->parsingEnvironmentFile()->needsUpdate())
-      ICore::self()->languageController()->backgroundParser()->addDocument(doc->url());
+      ICore::self()->languageController()->backgroundParser()->addDocument(IndexedString(doc->url()));
 }
 
 void DUChain::documentClosed(IDocument* document)
@@ -1496,17 +1517,19 @@ void DUChain::documentLoadedPrepare(KDevelop::IDocument* doc)
 {
   if(sdDUChainPrivate->m_destroyed)
     return;
+
+  const IndexedString url(doc->url());
   DUChainWriteLocker lock( DUChain::lock() );
   QMutexLocker l(&sdDUChainPrivate->m_chainsMutex);
 
   TopDUContext* standardContext = DUChainUtils::standardContextForUrl(doc->url());
-  QList<TopDUContext*> chains = chainsForDocument(doc->url());
+  QList<TopDUContext*> chains = chainsForDocument(url);
 
   QList<KDevelop::ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(doc->url());
 
   if(standardContext) {
     Q_ASSERT(chains.contains(standardContext)); //We have just loaded it
-    Q_ASSERT((standardContext->url().toUrl() == doc->url()));
+    Q_ASSERT((standardContext->url() == url));
 
     sdDUChainPrivate->m_openDocumentContexts.insert(standardContext);
 
@@ -1537,13 +1560,13 @@ void DUChain::documentLoadedPrepare(KDevelop::IDocument* doc)
     }
     
     if(needsUpdate || !(standardContext->features() & TopDUContext::AllDeclarationsContextsAndUses)) {
-      ICore::self()->languageController()->backgroundParser()->addDocument(doc->url(), (TopDUContext::Features)(TopDUContext::AllDeclarationsContextsAndUses | TopDUContext::ForceUpdate));
+      ICore::self()->languageController()->backgroundParser()->addDocument(IndexedString(doc->url()), (TopDUContext::Features)(TopDUContext::AllDeclarationsContextsAndUses | TopDUContext::ForceUpdate));
       return;
     }
   }
     
   //Add for highlighting etc.
-  ICore::self()->languageController()->backgroundParser()->addDocument(doc->url(), TopDUContext::AllDeclarationsContextsAndUses);
+  ICore::self()->languageController()->backgroundParser()->addDocument(IndexedString(doc->url()), TopDUContext::AllDeclarationsContextsAndUses);
 }
 
 void DUChain::documentRenamed(KDevelop::IDocument* doc)
@@ -1555,7 +1578,7 @@ void DUChain::documentRenamed(KDevelop::IDocument* doc)
     ///Maybe this happens when a file was deleted?
     kWarning() << "Strange, url of renamed document is invalid!";
   }else{
-    ICore::self()->languageController()->backgroundParser()->addDocument(doc->url(), (TopDUContext::Features)(TopDUContext::AllDeclarationsContextsAndUses | TopDUContext::ForceUpdate));
+    ICore::self()->languageController()->backgroundParser()->addDocument(IndexedString(doc->url()), (TopDUContext::Features)(TopDUContext::AllDeclarationsContextsAndUses | TopDUContext::ForceUpdate));
   }
 }
 
@@ -1569,8 +1592,12 @@ Definitions* DUChain::definitions()
   return &sdDUChainPrivate->m_definitions;
 }
 
-void DUChain::aboutToQuit()
+void DUChain::shutdown()
 {
+  // if core is not shutting down, we can end up in deadlocks or crashes
+  // since language plugins might still try to access static duchain stuff
+  Q_ASSERT(!ICore::self() || ICore::self()->shuttingDown());
+
   QMutexLocker lock(&sdDUChainPrivate->cleanupMutex());
 
   {
@@ -1641,7 +1668,8 @@ void DUChain::refCountDown(TopDUContext* top) {
     sdDUChainPrivate->m_referenceCounts.remove(top);
 }
 
-void DUChain::emitDeclarationSelected(DeclarationPointer decl) {
+void DUChain::emitDeclarationSelected(const DeclarationPointer& decl)
+{
   emit declarationSelected(decl);
 }
 
@@ -1650,7 +1678,7 @@ KDevelop::ReferencedTopDUContext DUChain::waitForUpdate(const KDevelop::IndexedS
 
   WaitForUpdate waiter;
   
-  waiter.m_dataMutex.lock();
+  waiter.m_dataMutex.lockInline();
   
   {
     DUChainReadLocker readLock(DUChain::lock());
@@ -1661,6 +1689,12 @@ KDevelop::ReferencedTopDUContext DUChain::waitForUpdate(const KDevelop::IndexedS
 //   waiter.m_waitMutex.lock();
 //   waiter.m_dataMutex.unlock();
   while(!waiter.m_ready) {
+    // we might have been shut down in the meanwhile
+    if (!ICore::self()) {
+      return 0;
+    }
+
+    QMetaObject::invokeMethod(ICore::self()->languageController()->backgroundParser(), "parseDocuments");
     QApplication::processEvents();
     usleep(1000);
   }
@@ -1682,7 +1716,7 @@ void DUChain::updateContextForUrl(const IndexedString& document, TopDUContext::F
     QMetaObject::invokeMethod(notifyReady, "updateReady", Qt::DirectConnection, Q_ARG(KDevelop::IndexedString, document), Q_ARG(KDevelop::ReferencedTopDUContext, ReferencedTopDUContext(standardContext)));
   }else{
     ///Start a parse-job for the given document
-    ICore::self()->languageController()->backgroundParser()->addDocument(document.toUrl(), minFeatures, priority, notifyReady);
+    ICore::self()->languageController()->backgroundParser()->addDocument(document, minFeatures, priority, notifyReady);
   }
 }
 
