@@ -30,6 +30,8 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QDateTime>
+#include <QFileSystemWatcher>
+#include <QTimer>
 
 #include <interfaces/icore.h>
 #include <interfaces/iprojectcontroller.h>
@@ -51,6 +53,7 @@
 #include <KMessageBox>
 #include <KStandardDirs>
 #include <KTextEdit>
+#include <KDirWatch>
 #include "gitjob.h"
 #include "gitmessagehighlighter.h"
 
@@ -139,7 +142,7 @@ QString revisionInterval(const KDevelop::VcsRevision& rev, const KDevelop::VcsRe
 {
     QString ret;
     if(rev.revisionType()==VcsRevision::Special &&
-                rev.revisionValue().value<VcsRevision::RevisionSpecialType>()==VcsRevision::Start) //if we want it to the begining just put the revisionInterval
+                rev.revisionValue().value<VcsRevision::RevisionSpecialType>()==VcsRevision::Start) //if we want it to the beginning just put the revisionInterval
         ret = toRevisionName(limit, QString());
     else {
         QString dst = toRevisionName(limit);
@@ -182,13 +185,16 @@ GitPlugin::GitPlugin( QObject *parent, const QVariantList & )
     KDEV_USE_EXTENSION_INTERFACE( KDevelop::IBranchingVersionControl )
 
     m_hasError = false;
-    core()->uiController()->addToolView(i18n("Git"), dvcsViewFactory());
     setObjectName("Git");
     
     DVcsJob* versionJob = new DVcsJob(QDir::tempPath(), this, KDevelop::OutputJob::Silent);
     *versionJob << "git" << "--version";
     connect(versionJob, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitVersionOutput(KDevelop::DVcsJob*)));
     ICore::self()->runController()->registerJob(versionJob);
+    
+    m_watcher = new KDirWatch(this);
+    connect(m_watcher, SIGNAL(dirty(QString)), SLOT(fileChanged(QString)));
+    connect(m_watcher, SIGNAL(created(QString)), SLOT(fileChanged(QString)));
 }
 
 GitPlugin::~GitPlugin()
@@ -252,12 +258,6 @@ DVcsJob* GitPlugin::errorsFound(const QString& error, KDevelop::OutputJob::Outpu
     return j;
 }
 
-void GitPlugin::unload()
-{
-    core()->uiController()->removeToolView( dvcsViewFactory() );
-}
-
-
 QString GitPlugin::name() const
 {
     return QLatin1String("Git");
@@ -272,7 +272,7 @@ bool GitPlugin::isValidDirectory(const KUrl & dirPath)
 {
     QDir dir=dotGitDirectory(dirPath);
 
-    return dir.exists(".git");
+    return dir.cd(".git") && dir.exists("HEAD");
 }
 
 bool GitPlugin::isVersionControlled(const KUrl &path)
@@ -489,7 +489,7 @@ VcsJob* GitPlugin::log(const KUrl& localLocation,
 {
     DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
     job->setType(VcsJob::Log);
-    *job << "git" << "log" << "--date=raw" /*<< "--numstat"*/;
+    *job << "git" << "log" << "--date=raw" << "--name-status" << "-M80%" << "--follow";
     QString rev = revisionInterval(dst, src);
     if(!rev.isEmpty())
         *job << rev;
@@ -503,7 +503,10 @@ VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& r
 {
     DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
     job->setType(VcsJob::Log);
-    *job << "git" << "log" << "--date=raw" /*<< "--numstat"*/ << toRevisionName(rev, QString());
+    *job << "git" << "log" << "--date=raw" << "--name-status" << "-M80%" << "--follow";
+    QString revStr = toRevisionName(rev, QString());
+    if(!revStr.isEmpty())
+        *job << revStr;
     if(limit>0)
         *job << QString("-%1").arg(limit);
     
@@ -651,7 +654,7 @@ VcsJob* GitPlugin::renameBranch(const KUrl& repository, const QString& oldBranch
 VcsJob* GitPlugin::currentBranch(const KUrl& repository)
 {
     DVcsJob* job = new DVcsJob(urlDir(repository), this, OutputJob::Silent);
-    *job << "git" << "symbolic-ref" << "HEAD";
+    *job << "git" << "rev-parse" << "--abbrev-ref" << "HEAD";
     connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitCurrentBranch(KDevelop::DVcsJob*)));
     return job;
 }
@@ -659,7 +662,13 @@ VcsJob* GitPlugin::currentBranch(const KUrl& repository)
 void GitPlugin::parseGitCurrentBranch(DVcsJob* job)
 {
     QString out = job->output().trimmed();
-    job->setResults(out.right(out.size()-11));
+
+    // in detached state, we'd like to return an empty string
+    if (out == "HEAD") {
+        out.clear();
+    }
+
+    job->setResults(out);
 }
 
 VcsJob* GitPlugin::branches(const KUrl &repository)
@@ -681,7 +690,11 @@ void GitPlugin::parseGitBranchOutput(DVcsJob* job)
         // "git rev-list" chokes on these entries and we do not need duplicate branches altogether.
         if (branch.contains("->"))
             continue;
-        
+
+        // Skip entries such as '(no branch)'
+        if (branch.contains("(no branch)"))
+            continue;
+
         if (branch.startsWith('*'))
             branch = branch.right(branch.size()-2);
         
@@ -946,11 +959,24 @@ void GitPlugin::parseLogOutput(const DVcsJob * job, QList<DVcsEvent>& commits) c
     }
 }
 
+VcsItemEvent::Actions actionsFromString(char c)
+{
+    switch(c) {
+        case 'A': return VcsItemEvent::Added;
+        case 'D': return VcsItemEvent::Deleted;
+        case 'R': return VcsItemEvent::Replaced;
+        case 'M': return VcsItemEvent::Modified;
+    }
+    return VcsItemEvent::Modified;
+}
+
 void GitPlugin::parseGitLogOutput(DVcsJob * job)
 {
     static QRegExp commitRegex( "^commit (\\w{8})\\w{32}" );
     static QRegExp infoRegex( "^(\\w+):(.*)" );
-    static QRegExp modificationsRegex("(\\d+)\\s+(\\d+)\\s+(.+)"); //18      2       shell/workingsets/workingsetwidget.cpp
+    static QRegExp modificationsRegex("^([A-Z])[0-9]*\t([^\t]+)\t?(.*)", Qt::CaseSensitive, QRegExp::RegExp2);
+    //R099    plugins/git/kdevgit.desktop     plugins/git/kdevgit.desktop.cmake
+    //M       plugins/grepview/CMakeLists.txt
 
     QList<QVariant> commits;
 
@@ -975,6 +1001,7 @@ void GitPlugin::parseGitLogOutput(DVcsJob * job)
             if (pushCommit) {
                 item.setMessage(message.trimmed());
                 commits.append(QVariant::fromValue(item));
+                item.setItems(QList<VcsItemEvent>());
             } else {
                 pushCommit = true;
             }
@@ -990,17 +1017,16 @@ void GitPlugin::parseGitLogOutput(DVcsJob * job)
                 item.setDate(QDateTime::fromTime_t(infoRegex.cap(2).trimmed().split(' ')[0].toUInt()));
             }
         } else if (modificationsRegex.exactMatch(line)) {
-            int additions = modificationsRegex.cap(1).toUInt();
-            int removals = modificationsRegex.cap(2).toUInt();
-            QString filename = modificationsRegex.cap(3);
-            
-            VcsItemEvent::Actions a;
-            if(additions>0 || removals>0)
-                a=VcsItemEvent::Modified;
+            VcsItemEvent::Actions a = actionsFromString(modificationsRegex.cap(1)[0].toAscii());
+            QString filenameA = modificationsRegex.cap(2);
             
             VcsItemEvent itemEvent;
             itemEvent.setActions(a);
-            itemEvent.setRepositoryLocation(filename);
+            itemEvent.setRepositoryLocation(filenameA);
+            if(a==VcsItemEvent::Replaced) {
+                QString filenameB = modificationsRegex.cap(3);
+                itemEvent.setRepositoryCopySourceLocation(filenameB);
+            }
             
             item.addItem(itemEvent);
         } else if (line.startsWith("    ")) {
@@ -1192,9 +1218,11 @@ VcsStatusInfo::State GitPlugin::messageToState(const QString& msg)
             ret = VcsStatusInfo::ItemAdded;
             break;
         case 'R':
-        case 'C':
             ret = VcsStatusInfo::ItemModified;
             break;
+        case 'C':
+            ret = VcsStatusInfo::ItemHasConflicts;
+            break;
         case ' ':
             ret = msg[1] == 'M' ? VcsStatusInfo::ItemModified : VcsStatusInfo::ItemDeleted;
             break;
@@ -1346,3 +1374,28 @@ QString GitPlugin::errorDescription() const
 {
     return m_errorDescription;
 }
+
+void GitPlugin::registerRepositoryForCurrentBranchChanges(const KUrl& repository)
+{
+    QDir dir = dotGitDirectory(repository);
+    QString headFile = dir.absoluteFilePath(".git/HEAD");
+    m_watcher->addFile(headFile);
+}
+
+void GitPlugin::fileChanged(const QString& file)
+{
+    Q_ASSERT(file.endsWith("HEAD"));
+    KUrl fileUrl(KUrl::fromPath(file));
+    fileUrl = fileUrl.upUrl(); //SMTH/.git/HEAD -> SMTH/.git/
+    fileUrl = fileUrl.upUrl(); //SMTH/.git/ -> SMTH/
+    
+    //We need to delay the emitted signal, otherwise the branch hasn't change yet
+    //and the repository is not functional
+    m_branchesChange.append(fileUrl);
+    QTimer::singleShot(1000, this, SLOT(delayedBranchChanged()));
+}
+
+void GitPlugin::delayedBranchChanged()
+{
+    emit repositoryBranchChanged(m_branchesChange.takeFirst());
+}
