@@ -367,7 +367,7 @@ void LaunchConfigurationDialog::selectionChanged(QItemSelection selected, QItemS
             deleteConfig->setEnabled( false );
             stack->setCurrentIndex( 0 );
             QLabel* l = new QLabel(i18n("<i>Select a configuration to edit from the left,<br>"
-                                        "or click the \"Add new\" button to add a new one.</i>"), stack);
+                                        "or click the \"Add New\" button to add a new one.</i>"), stack);
             l->setAlignment(Qt::AlignCenter);
             stack->addWidget(l);
             stack->setCurrentWidget(l);
